#include "../include/aprecv-general.h"

int parseoptions (int argc, char *argv[]) {
  static struct gengetopt_args_info args_info;
  static int opt_filter;
  
  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
  struct stat fstats;
  static struct apsr_aprecv_module_info module_info;
  #endif

  if(cmd_parser (argc, argv, &args_info) != 0) {
    apsr_stream_output((FILE *) stderr, "Error: Unable to parse the command line!\n");
    exit(EXIT_FAILURE);
  }

  if(args_info.help_given) usage(argv[0], "");
  if(args_info.version_given) showversion();


  /* Quiet mode */
  if(args_info.quiet_given)		apsr_set_quiet();
  if(args_info.really_quiet_given)	apsr_set_rquiet();
 
  if(!args_info.max_packets_given) args_info.max_packets_arg = -1;
  if(!args_info.server_port_given) args_info.server_port_arg = APSRSERVER_PORT;
  if(args_info.snaplen_arg >= 65535) {
	  apsr_output("Error: Snaplen(%u) is to big, changed it to max(65535).\n", args_info.snaplen_arg);
          args_info.snaplen_arg = 65535;
  }
  else if(!args_info.snaplen_given)     
	  args_info.snaplen_arg     = 65535;

  if(args_info.expand_all_given)	expand_all=1;
  if(args_info.expand_ethernet_given)	expand_ethernet=1;
  if(args_info.expand_llc_given)	expand_llc=1;
  if(args_info.expand_token_given)	expand_token=1;
  if(args_info.expand_fddi_given)	expand_fddi=1;
  if(args_info.expand_arp_given)	expand_arp=1;
  if(args_info.expand_pppoe_given)	expand_pppoe=1;

  if(args_info.no_dissection_given)	no_dissection=1;

  if(args_info.count_given)		count_bytes=1;
  if(args_info.verbose_given)		verbose=1;
  if(args_info.calc_checksums_given)	calc_checksums=1;
  if(args_info.statistic_given)		statistic=1;
  if(args_info.beep_on_given)		beep_on=1;
  if(args_info.daemon_given)		aprecv_daemon=1;
  
  if(args_info.print_raw_hex_given)	print_raw_hex=1;
  if(args_info.print_ip_hex_given)	print_payload_ip_hex=1;
  if(args_info.print_pppoe_hex_given)	print_payload_pppoe_hex=1;
  if(args_info.print_udp_hex_given)	print_payload_udp_hex=1;
  if(args_info.print_tcp_hex_given)	print_payload_tcp_hex=1;
  if(args_info.print_tcp_text_given)	print_payload_tcp_text=1;
  if(args_info.print_udp_text_given)	print_payload_udp_text=1;
  if(args_info.print_tcp_stream_given)	print_tcp_stream=1;

  if(args_info.use_apsrlib_given) 	use_apsrlib=1;

  #ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
  if(args_info.bt_given)		use_apsrlib_bluetooth=1;
  #endif
  
  if(args_info.logfile_given) {
    aprecv_log_file=xmalloc(strlen(args_info.logfile_arg)+1);
    strncpy(aprecv_log_file, args_info.logfile_arg, strlen(args_info.logfile_arg));
  }

  if(args_info.logfile_raw_given) {
    aprecv_log_raw_file=xmalloc(strlen(args_info.logfile_raw_arg)+1);
    strncpy(aprecv_log_raw_file, args_info.logfile_raw_arg, strlen(args_info.logfile_raw_arg));
  }

  if(args_info.device_given) {
    device=xmalloc(strlen(args_info.device_arg)+1);
    strncpy(device, args_info.device_arg, strlen(args_info.device_arg));
  }

  if(args_info.pcap_file_given) {
    pcap_log_file=xmalloc(strlen(args_info.pcap_file_arg)+1);
    memset(pcap_log_file, '\0', sizeof(pcap_log_file));
    strncpy(pcap_log_file, args_info.pcap_file_arg, strlen(args_info.pcap_file_arg));
  }

  if(args_info.iptrace_file_given) {
    iptrace_trace_file=xmalloc(strlen(args_info.iptrace_file_arg)+1);
    memset(iptrace_trace_file, '\0', sizeof(iptrace_trace_file));
    strncpy(iptrace_trace_file, args_info.iptrace_file_arg, strlen(args_info.iptrace_file_arg));
  }
  
  if(args_info.snoop_file_given) {
    snoop_trace_file=xmalloc(strlen(args_info.snoop_file_arg)+1);
    memset(snoop_trace_file, '\0', sizeof(snoop_trace_file));
    strncpy(snoop_trace_file, args_info.snoop_file_arg, strlen(args_info.snoop_file_arg));
  }
  
  if(args_info.server_addr_given) {
    server_addr=xmalloc(strlen(args_info.server_addr_arg)+1);
    strncpy(server_addr, args_info.server_addr_arg, strlen(args_info.server_addr_arg));
  }

  /* We MUST load the modules BEFORE we set the (pseudo) BPF filter, */
  /* because the modules can set their own BPF filters!              */
  /* Command line filters will NOT be overwritten by module filters! */
  /* The --module-ignore option will ignore the builtin BPF filter   */
  /* of the module.                                                  */
  if(args_info.module_given) {
  	  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
	  load_module(args_info.module_arg);
	  module_name = xmalloc(strlen(args_info.module_arg)+1);
	  strncpy(module_name, args_info.module_arg, strlen(args_info.module_arg));

	  module_info=get_module_info();
	  
	  if(args_info.module_ignore_given) {
		  apsr_output("Warning: Ignoring modules' builtin BPF filter!\n\n");
	  } else {
	    if((!args_info.filter_given && !args_info.filter_nopt_given) && module_info.bpf_filter) {
	    	  args_info.filter_nopt_arg = xmalloc(strlen(module_info.bpf_filter)+1);
		  strncpy(args_info.filter_nopt_arg, module_info.bpf_filter, strlen(module_info.bpf_filter));
		  apsr_output("Filter: %s\n", module_info.bpf_filter);
	    }
	  }
  	#else
  	ERROR("Module support is not available under this operating system!");
	ERROR("No modules were loaded!\n");
	#endif
  }
  else if (args_info.modules_given) {
          #ifdef HAVE_APSRLIB_MODULE_SUPPORT
	  lstat(args_info.modules_arg, &fstats);
	  if(!S_ISDIR(fstats.st_mode)) {
		 ERROR("Your given argument to --modules isn't a directory! No modules were loaded!\n");
	  }
	  else {
		 load_modules_in_path(args_info.modules_arg);
		 modules_in_path = 1;
	  }
          #else
          ERROR("Module support is not available under this operating system!");
          ERROR("No modules were loaded!\n");
          #endif
  }
  else {
	  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
	  modules_in_path = 0;
          #endif
  }
	  
  if(args_info.filter_given && args_info.filter_nopt_given) usage(argv[0], "Use only optimized or not optimized filter rules.");

  if(args_info.filter_given) opt_filter=1;
  else     opt_filter=0;

  if(args_info.device_given && args_info.pcap_file_given && args_info.iptrace_file_given && args_info.snoop_file_given) usage(argv[0], "Don't use -d and -P | -I together.");

  if(args_info.use_apsrlib_given && (args_info.pcap_file_given || args_info.filter_given)) 
	  usage(argv[0], "Currently not supported by APSRLib.");

  if(!args_info.device_given && args_info.use_apsrlib_given)
	   usage(argv[0], "Please enter a device to use.");

  if(args_info.daemon_given) daemon_mode();

  if(args_info.logfile_given && !args_info.daemon_given) {
	if(!(freopen(args_info.logfile_arg, "a", stdout))) {
		apsr_stream_output((FILE *) stderr, "Error: Can't open logfile %s.\n", 
				args_info.logfile_arg);
                #ifdef APRECV_DEBUG
		apsr_stream_output((FILE *) stderr, "freopen %s.\n", strerror(errno));
		#endif
	}

	#ifdef HAVE_APSRLIB_GET_TIME_FORMATTED
	apsr_output("Starting logging on %s(local time, not GMT)\n", 
			apsr_get_time_formatted());
	#endif
  }

  if(args_info.statistic_given) {
  	  #ifdef HAVE_APSRLIB_GET_TIME_FORMATTED
	  start_receive_time = time(NULL);
	  start_time=apsr_get_time_formatted();
	  #endif
  }
  
  if(args_info.server_addr_given) 
	  open_server_connection(args_info.server_addr_arg, args_info.server_port_arg);

  #ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
  if(args_info.bt_given)
	  start_sniffing_bluetooth(argv[0], args_info.bt_arg);
  
  else if(args_info.filter_given && !args_info.use_apsrlib_given) 
	  start_sniffing_pcap(argv[0], args_info.promisc_flag, args_info.snaplen_arg, args_info.filter_arg, opt_filter, args_info.max_packets_arg);
  #else 
  if(args_info.filter_given && !args_info.use_apsrlib_given)
	  start_sniffing_pcap(argv[0], args_info.promisc_flag, args_info.snaplen_arg, args_info.filter_arg, opt_filter, args_info.max_packets_arg);
  #endif
  else if(args_info.use_apsrlib_given) 
	  start_sniffing_apsr(argv[0], args_info.promisc_flag, args_info.snaplen_arg, args_info.max_packets_arg);
  else if(args_info.iptrace_file_given) 
	  read_iptrace(args_info.max_packets_arg);
  else if(args_info.snoop_file_given)
          read_snoop(args_info.max_packets_arg);
  else 
	  start_sniffing_pcap(argv[0], args_info.promisc_flag, args_info.snaplen_arg, args_info.filter_nopt_arg, opt_filter, args_info.max_packets_arg);

  return(RET_SUCCESS);
}
