#include "../include/aprecv-general.h"

static const char *arpopcodes[]= {
	" (Unknown)",
	" (ARP request)",
	" (ARP reply)",
	" (Reverse ARP request)",
	" (Reverse ARP reply)",
	" (Dynamic Reverse ARP request)",
	" (Dynmaic Reverse ARP reply)",
	" (Dynmaic Reverse ARP error)",
	" (Inverse ARP request)",
	" (Inverse ARP reply)",
	" (ATM ARP NAK)",
	" (MARS-Request)",
	" (MARS-Multi)", 
	" (MARS-MServ)",
	" (MARS-Join)",
	" (MARS-Leave)",
	" (MARS-Nak)",
	" (MARS-UnServ)",
	" (MARS-SJoin)",
	" (MARS-SLeave)",
	" (MARS-Grouplist-Request)",
	" (MARS-Grouplist-Reply)",
	" (MARS-Redirect-Map)",
	" (MAPOS-UNARP)",
};

static const char *arphwtypes[]= {
	" (Unknown)",
	" (Ethernet 10 Mbit/FDDI)",
	" (Amateur Radio AX.25)",
	" (Proteon Proton Token Ring)",
	" (Chaos)",
	" (IEEE 802.x networks)",
	" (ARCnet)",
	" (HyperChannel)",
	" (LanStar)",
	" (Autonet Short Address)",
	" (LocalTalk)",
        " (LoaclNet)",
	" (Ultra Link)", 
	" (SMDS)",
	" (FrameRelay)",
	" (ATM)",
	" (HDLC)",
	" (FiberChannel)",
	" (ATM)",
	" (Serial Line)",
	" (ATM)",
	" (MIL-STD-188-220)",
	" (Metricom)",
	" (IEEE 1394.1995)",
	" (MAPOS)",
	" (Twinaxial)",
	" (EUI-64)",
	" (HIPARP)",
};

int print_arp_packet (unsigned char *packet, int opt_size) {
  struct arp_pkt_header *arp_header;
  unsigned short hardware_type, protocol_type, opcode;
  unsigned char *temp_buf;
  unsigned char sender_hw_addr[APSR_ETHER_ADDR_LEN], sender_pr_addr[4], target_hw_addr[APSR_ETHER_ADDR_LEN], target_pr_addr[4];

  arp_header = (struct arp_pkt_header *)(packet+data_link_size+opt_size);
	
  hardware_type = apsr_ntohs(arp_header->arp_hat);
  protocol_type = apsr_ntohs(arp_header->arp_pat);
  opcode 	= apsr_ntohs(arp_header->arp_oc);
  
  apsr_output("\n- ARP Header -\n");

  if(expand_arp || expand_all) {  
    apsr_output("  Hardware Type: %d",(int) hardware_type);
  
    if(verbose)
	  apsr_output("%s", arphwtypes[hardware_type]);
		  
    apsr_output("\n  Protocol Type: %d",(int) protocol_type);

    if(verbose) {
	  switch(protocol_type) {
		  case APSR_ETHERTYPE_IP:
		    apsr_output(" (IPv4)");	/* IPv4 */
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	  }
    }   
  
    apsr_output("\n  Hardware Len: %d, Protocol Len: %d, Opcode: %d", arp_header->arp_hal, arp_header->arp_pal,(int) opcode); 

    if(verbose) 
	    apsr_output("%s", arpopcodes[opcode]);
  
    apsr_output("\n  Sender Hardware Addr: ");
    temp_buf = (packet+data_link_size+opt_size+APSR_ARP_HDR_LEN);
    if(arp_header->arp_hal != 0 && arp_header->arp_hal == APSR_ETHER_ADDR_LEN) {
      sender_hw_addr[0] = temp_buf[0];
      sender_hw_addr[1] = temp_buf[1];
      sender_hw_addr[2] = temp_buf[2];
      sender_hw_addr[3] = temp_buf[3];
      sender_hw_addr[4] = temp_buf[4];
      sender_hw_addr[5] = temp_buf[5];
      apsr_output("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
		      sender_hw_addr[0], sender_hw_addr[1],
		      sender_hw_addr[2], sender_hw_addr[3],
		      sender_hw_addr[4], sender_hw_addr[5]);
      if(verbose)
	      resolve_mac(sender_hw_addr);
    }    
    else if(arp_header->arp_hal != APSR_ETHER_ADDR_LEN) 
      apsr_output("<Not implemented @ the moment>");
    else
      apsr_output("(None given, because hardware length is zero)");
    
    apsr_output("\n  Sender Protocol Addr: ");
    if(protocol_type == APSR_ETHERTYPE_IP && arp_header->arp_pal == 4 && arp_header->arp_pal != 0) {
      sender_pr_addr[0] = temp_buf[6];
      sender_pr_addr[1] = temp_buf[7];
      sender_pr_addr[2] = temp_buf[8];
      sender_pr_addr[3] = temp_buf[9];
      apsr_output("%u.%u.%u.%u",
		      sender_pr_addr[0], sender_pr_addr[1],
		      sender_pr_addr[2], sender_pr_addr[3]);
/*    	  if(verbose) 
	    resolve_ip(sender_pr_addr); */
    }
    else if(protocol_type != APSR_ETHERTYPE_IP && arp_header->arp_pal != 4) 
      apsr_output("<Not implemented @ the moment>");
    else
      apsr_output("(None given, because protocol length is zero)"); 
    
    apsr_output("\n  Target Hardware Addr: ");
    if(arp_header->arp_hal != 0 && arp_header->arp_hal == APSR_ETHER_ADDR_LEN) {    
      target_hw_addr[0] = temp_buf[10];
      target_hw_addr[1] = temp_buf[11];
      target_hw_addr[2] = temp_buf[12];
      target_hw_addr[3] = temp_buf[13];
      target_hw_addr[4] = temp_buf[14];
      target_hw_addr[5] = temp_buf[15];
      apsr_output("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
		      target_hw_addr[0], target_hw_addr[1],
		      target_hw_addr[2], target_hw_addr[3],
		      target_hw_addr[4], target_hw_addr[5]);
      if(verbose)
	      resolve_mac(target_hw_addr);
    }
    else if(arp_header->arp_hal != APSR_ETHER_ADDR_LEN)
      apsr_output("<Not implemented @ the moment>");
    else
      apsr_output("(None given, because hardware length is zero)");
    
    apsr_output("\n  Target Protocol Addr: ");
    if(protocol_type == APSR_ETHERTYPE_IP && arp_header->arp_pal == 4 && arp_header->arp_pal != 0) {
      target_pr_addr[0] = temp_buf[16];
      target_pr_addr[1] = temp_buf[17];
      target_pr_addr[2] = temp_buf[18];
      target_pr_addr[3] = temp_buf[19];
      apsr_output("%u.%u.%u.%u",
		      target_pr_addr[0], target_pr_addr[1],
		      target_pr_addr[2], target_pr_addr[3]);
/*        if(verbose)
 *                    resolve_ip(sender_pr_addr); */
    }
    else if(protocol_type != APSR_ETHERTYPE_IP && arp_header->arp_pal != 4)
      apsr_output("<Not implemented @ the moment>");
    else 
      apsr_output("(None given, because protocol length is zero)");
    apsr_output("\n");
 
    if((hardware_type == APSR_ARP_HWTYPE_E10 || hardware_type == APSR_ARP_HWTYPE_IEEE) && arp_header->arp_hal != 6)  {
	    apsr_output("  Error: Invalid length for Ethernet/IEEE 802.x networks\n");

	    if(statistic)
		    stats.err.arp++;
    }
  }
  
  if(statistic) 
        stats.c.arp++;
 
  return(RET_SUCCESS);
}
