#include "../include/aprecv-general.h"

static const char *egp_types[]= {
	" (Unknown)",
	" (Update)",
	" (Poll)",
	" (Acquire)",
	" (Reach)",
	" (Error)",
};

int print_egp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct egp_pkt_header *egp_header;
  static unsigned short pkt_checksum, correct_checksum;
  static char *ip_source;
 
  egp_header = (struct egp_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("- EGP Header -\n  Version: %d\n",(int) egp_header->version);
  if(!verbose) 
	  apsr_output("  Type: %d, Code: %d\n",(int) egp_header->type,(int) egp_header->code);
  else {
	  apsr_output("  Type: %d%s",
			  (int) egp_header->type, 
			  egp_types[egp_header->type]);
         apsr_output("\n  Code: %d",(int) egp_header->code);
	 if(egp_header->type == APSR_EGP_TYPE_NEIGHBOR_ACQ_MES) {
		 switch(egp_header->code) {
			 case APSR_EGP_CODE_REQUEST_COMMAND:
		           apsr_output(" (Request)");
			   break;
			 case APSR_EGP_CODE_CONFIRM_RESPONSE:
			   apsr_output(" (Confirm)");
			   break;
			 case APSR_EGP_CODE_REFUSE_RESPONSE:
			   apsr_output(" (Refuse)");
			   break;
			 case APSR_EGP_CODE_CEASE_COMMAND:
			   apsr_output(" (Cease)");
			   break;
			 case APSR_EGP_CODE_CEASE_ACK_RESP:
			   apsr_output(" (Cease Ack)");
			   break;
			 default:
			   apsr_output(" (Unkown)");
			   break;
		 }
         }		 
	 else if(egp_header->type == APSR_EGP_TYPE_NEIGHBOR_REA_MES) {
                 switch(egp_header->code) {
                         case APSR_EGP_CODE_HELLO_COMMAND:
	                   apsr_output(" (Hello)");
	                   break;
			 case APSR_EGP_CODE_I_H_U_RESPONSE:
			   apsr_output(" (I-H-U)");
			   break;
			 default:
			   apsr_output(" (Unkown)");
			   break;
		 }
	 }
	 apsr_output("\n");
  }		 
  
  pkt_checksum = apsr_ntohs(egp_header->sum);
  if(!verbose)
	  apsr_output("  Status: %d, Checksum: 0x%04x",(int) egp_header->status,
			  pkt_checksum);
  else {
	  apsr_output("  Status: %d",(int) egp_header->status);
          if(egp_header->type == APSR_EGP_TYPE_NEIGHBOR_ACQ_MES) {
	    if(egp_header->code == APSR_EGP_CODE_CONFIRM_RESPONSE) {
		  switch(egp_header->status) {
			  case APSR_EGP_STATUS_UNSPEC:
			    apsr_output(" (Unspecified)");
			    break;
			  case APSR_EGP_STATUS_ACTIVE_MODE:
			    apsr_output(" (Active)");
			    break;
			  case APSR_EGP_STATUS_PASSIVE_MODE:
			    apsr_output(" (Passive)");
			    break;
			  default: 
			    apsr_output(" (Unkown)");
			    break;
		 }
	    }
	    else if(egp_header->code == APSR_EGP_CODE_CEASE_ACK_RESP) {
		 switch(egp_header->status) {
		         case APSR_EGP_STATUS_UNSPEC:
		           apsr_output(" (Unspecified)");
		           break;
         	         case APSR_EGP_STATUS_INSUF_RESOURCES:
			   apsr_output(" (Insufficient Resources)");
			   break;
			 case APSR_EGP_STATUS_ADMIN_PROHIB:
			   apsr_output(" (Administratively prohibited)");
			   break;
			 case APSR_EGP_STATUS_GOING_DOWN:
			   apsr_output(" (Going down)");
			   break;
			 case APSR_EGP_STATUS_PARA_PROBLEM:
			   apsr_output(" (Parameter Problem)");
			   break;
			 case APSR_EGP_STATUS_PROTO_VIOLATION:
			   apsr_output(" (Protocol Violation)");
			   break;
			 default:
			   apsr_output(" (Unknown)");
			   break;
		 }	 
	    }	 
	  }
	  else if(egp_header->type == APSR_EGP_TYPE_NEIGHBOR_REA_MES && 
			  (egp_header->code == APSR_EGP_CODE_HELLO_COMMAND ||
			   egp_header->code == APSR_EGP_CODE_I_H_U_RESPONSE)) { 
		  switch(egp_header->status) {
			  case APSR_EGP_STATUS_UP_STATE:
			    apsr_output(" (Up)");
			    break;
			  case APSR_EGP_STATUS_DOWN_STATE:
			    apsr_output(" (Down)");
			    break;
			  default:
			    apsr_output(" (Unknown)");
			    break;
 		  }  
          }
	  else if(egp_header->type == APSR_EGP_TYPE_POLL_COMMAND || 
			  egp_header->type == APSR_EGP_TYPE_UPDATE_RES_IND ||
			  egp_header->type == APSR_EGP_TYPE_ERROR_RES_IND) {
                  switch(egp_header->status) {
			  case APSR_EGP_STATUS_UP_STATE:
                            apsr_output(" (Up)");
                            break;
                          case APSR_EGP_STATUS_DOWN_STATE:
                            apsr_output(" (Down)");
                            break;
			  case APSR_EGP_STATUS_UNSOL_MES_BIT:
			    apsr_output(" (Unsolicited)");
			    break;
                          default:
                            apsr_output(" (Unknown)");
                            break;
                 }
          }		 
	  apsr_output("\n  Checksum: 0x%04x", pkt_checksum);
  }	  

  if(calc_checksums) {
	  egp_header->sum=0x0000;
          correct_checksum=apsr_ntohs(in_cksum((unsigned short *)egp_header,
          sizeof(struct egp_pkt_header)+(header->len-data_link_size-opt_size),
   			              0));
          if(pkt_checksum == correct_checksum) {
		  apsr_output(" (correct checksum)");
          }
          else {
		  apsr_output(" (incorrect checksum - should be 0x%04x)",
                             correct_checksum);

                  if(statistic)
			  stats.cr.egp++;
         }
  }
  
  apsr_output("\n  Autonomus System: %d, Sequence: %d\n", apsr_ntohs(egp_header->auto_sys),
		apsr_ntohs(egp_header->sequence));  
	
  if(egp_header->type == APSR_EGP_TYPE_NEIGHBOR_ACQ_MES) {
    apsr_output("  Hello Interval: %u, Poll Interval: %u\n", 
                apsr_ntohs(egp_header->ext.nam.hello_int),
                apsr_ntohs(egp_header->ext.nam.poll_int));
  }
  else if(egp_header->type == APSR_EGP_TYPE_ERROR_RES_IND) {
    apsr_output("  Reason: %u\n", apsr_ntohs(egp_header->ext.error.reason));
  }
  else if(egp_header->type == APSR_EGP_TYPE_POLL_COMMAND) {
    ip_source = strdup(inet_ntoa(egp_header->ext.poll.source));
    apsr_output("  Reserved: %u, IP Source Network: %s\n",
                apsr_ntohs(egp_header->ext.poll.reserved),
                ip_source);
    free(ip_source);
  } 

  if(statistic)
	stats.c.egp++;
  
  return(RET_SUCCESS);
}
