#include "../include/aprecv-general.h"

int print_icmpv4_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct icmpv4_pkt_header *icmpv4_header;
  static unsigned short pkt_checksum, correct_checksum;
  static char *gateway_ip;
  
  icmpv4_header = (struct icmpv4_pkt_header *)(packet+data_link_size+opt_size);

  apsr_output("- ICMPv4 Header -\n  Type: %d",(int) icmpv4_header->type);
  
  if(verbose) {
	  switch(icmpv4_header->type) {
		  case APSR_ICMP_ECHO_REPLY:
	            apsr_output(" (Echo reply)");
		    break;
		  case APSR_ICMP_DEST_UNREA:
		    apsr_output(" (Destination unreachable)");
		    break;
		  case APSR_ICMP_SOUR_QUENC:
		    apsr_output(" (Source quench)");
		    break;
		  case APSR_ICMP_REDIRECT:
		    apsr_output(" (Redirect)");
		    break;
		  case APSR_ICMP_ALTER_HOST:
		    apsr_output(" (Alternate host)");
		    break;
		  case APSR_ICMP_ECHO_REQUE:
		    apsr_output(" (Echo request)");
		    break;
		  case APSR_ICMP_TIME_EXCEE:
		    apsr_output(" (Time exceeded)");
		    break;
		  case APSR_ICMP_PARA_PROBL:
		    apsr_output(" (Parameter problem)");
		    break;
		  case APSR_ICMP_TIME_REQUE:
		    apsr_output(" (Time request)");
		    break;
		  case APSR_ICMP_TIME_REPLY:
		    apsr_output(" (Time reply)");
		    break;
		  case APSR_ICMP_INFO_REQUE:
		    apsr_output(" (Information request)");
		    break;
		  case APSR_ICMP_INFO_REPLY:
		    apsr_output(" (Information reply)");
		    break;
		  case APSR_ICMP_MASK_REQUE:
		    apsr_output(" (Mask request)");
		    break;
		  case APSR_ICMP_MASK_REPLY:
	            apsr_output(" (Mask reply)");
		    break;
		  case APSR_ICMP_TRACEROUTE:
		    apsr_output(" (Traceroute)");
		    break;
		  case APSR_ICMP_DOMAIN_REQ:
		    apsr_output(" (Domain name request)");
		    break;
		  case APSR_ICMP_DOMAIN_REP:
		    apsr_output(" (Domain name reply)");
		    break;
		  case APSR_ICMP_SECU_FAILU:
		    apsr_output(" (Security failures)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	}
  }
  
  if(icmpv4_header->type == APSR_ICMP_MASK_REPLY)  
         apsr_output("\n  Subnet-Mask-Length: %d",(int) icmpv4_header->code);
  else
         apsr_output("\n  Code: %d",(int) icmpv4_header->code);
  
  if(verbose) {
         switch(icmpv4_header->type) {
                case APSR_ICMP_DEST_UNREA:
                  switch(icmpv4_header->code) {
                          case APSR_ICMP_CODE_NTWRK_UNRE:
                            apsr_output(" (Network unreachable)");
                            break;
			  case APSR_ICMP_CODE_HOST_UNREA:
			    apsr_output(" (Host unreachable)");
			    break;
			  case APSR_ICMP_CODE_PROTO_UNRE:
			    apsr_output(" (Protocol unreachable)");
			    break;
			  case APSR_ICMP_CODE_PORT_UNREA:
			    apsr_output(" (Port unreachable)");
			    break;
                          case APSR_ICMP_CODE_FRAG_NEED:
                            apsr_output(" (Fragmentation needed but Don't Fragment set)");
                            break;
                          case APSR_ICMP_CODE_SOURCE_ROU:
                            apsr_output(" (Source Route failed)");
                            break;
                          case APSR_ICMP_CODE_DEST_NTWRK:
                            apsr_output(" (Destination Network unknown)");
                            break;
                          case APSR_ICMP_CODE_DEST_HOST:
                            apsr_output(" (Destination Host unknown)");
                            break;
                          case APSR_ICMP_CODE_SOURCE_HST:
                            apsr_output(" (Source Host isolated)");
                            break;
			  case APSR_ICMP_CODE_DEST_N_PRO:
                            apsr_output(" (Destination Network administr. prohibited)");
                            break;
                          case APSR_ICMP_CODE_DEST_H_PRO:
                            apsr_output(" (Destination Host administr. prohibited)");
                            break;
                          case APSR_ICMP_CODE_NTWRK_UTOS:
                            apsr_output(" (Network unreachable for TOS)");
                            break;
                          case APSR_ICMP_CODE_HOST_UNTOS:
                            apsr_output(" (Host unreachable for TOS)");
                            break;
                          case APSR_ICMP_CODE_COM_PROHIB:
                            apsr_output(" (Communication adminstr. prohibited by filter)");
                            break;
                          case APSR_ICMP_CODE_HOST_PR_VI:
                            apsr_output(" (Host Precedence Violation)");
                            break;
                          case APSR_ICMP_CODE_PREC_CUTOF:
                            apsr_output(" (Precedence Cutoff in effect)");
                            break;
			  default:
                            apsr_output(" (unknown)");
			    break;
		  }
		  break;
		case APSR_ICMP_REDIRECT:
		  switch(icmpv4_header->code) {
			  case APSR_ICMP_CODE_RED_NET:
			    apsr_output(" (Redirect datagrams for Network)");
			    break;
			  case APSR_ICMP_CODE_RED_HOS:
			    apsr_output(" (Redirect datagrams for Host)");
			    break;
			  case APSR_ICMP_CODE_RED_T_N:
			    apsr_output(" (Redirect datagrams for ToS and Network)");
			    break;
			  case APSR_ICMP_CODE_RED_T_H:
			    apsr_output(" (Redirect datagrams for ToS and Host)");
			  default:
			    apsr_output(" (unknown)");
			    break;
		  }
                  break;
		case APSR_ICMP_PARA_PROBL:
		  switch(icmpv4_header->code) {
			  case APSR_ICMP_CODE_POINT_ERR:
			    apsr_output(" (Pointer indicates the error)");
			    break;
			  case APSR_ICMP_CODE_MISS_OPTION:
			    apsr_output(" (Missing a required option)");
			    break;
                          case APSR_ICMP_CODE_BAD_LENGTH:
			    apsr_output(" (Bad Length)");
			    break;
			  default:
			    apsr_output(" (unknown)");
			    break;
		  }
                  break;		  
		case APSR_ICMP_TIME_EXCEE:
		  switch(icmpv4_header->code) {
                          case APSR_ICMP_CODE_TIME_TRANSIT:
		            apsr_output(" (Time to Live exceeded in Transit)");
			    break;
			  case APSR_ICMP_CODE_TIME_FRAG_REA:
			    apsr_output(" (Fragment Reassembly Time Exceeded)");
			    break;
			  default:
			    apsr_output(" (unknown)");
			    break;
		  }
		  break;					    
		case APSR_ICMP_ALTER_HOST:
		  switch(icmpv4_header->code) {
			  case APSR_ICMP_CODE_ALTER_ADDRESS:
			    apsr_output(" (Alternate Address for Host)");
			    break;
			  default:
			    apsr_output(" (unknown)");
			    break;
	          }
		  break; 
		case APSR_ICMP_SECU_FAILU:
		  switch(icmpv4_header->code) {
			  case APSR_ICMP_CODE_BAD_SPI:
			    apsr_output(" (Bad SPI)");
			    break;
			  case APSR_ICMP_CODE_AUTH_FAILED:
			    apsr_output(" (Authentication failed)");
			    break;
			  case APSR_ICMP_CODE_DECO_FAILED:
			    apsr_output(" (Decompression failed)");
			    break;
			  case APSR_ICMP_CODE_DECR_FAILED:
			    apsr_output(" (Decryption failed)");
			    break;
			  case APSR_ICMP_CODE_NEED_AUTHENT:
			    apsr_output(" (Need authentication)");
			    break;
			  case APSR_ICMP_CODE_NEED_AUTHORI:
	  		    apsr_output(" (Need authorization)");
		  	    break;
			  default:
			    apsr_output(" (unknown)");
			    break;
		  }
		  break;
		default:
                  break;
         }		  
  }
  
  pkt_checksum = apsr_ntohs(icmpv4_header->sum);
  apsr_output("\n  Checksum: 0x%04x", pkt_checksum);
  
  if(calc_checksums) {
    icmpv4_header->sum = 0;
    correct_checksum=apsr_ntohs(in_cksum((unsigned short *)icmpv4_header, sizeof(struct icmpv4_pkt_header)+(header->len-data_link_size-opt_size), 0));
    if(pkt_checksum == correct_checksum) {
	    apsr_output(" (correct checksum)");
    }
    else {
            apsr_output(" (incorrect checksum - should be 0x%04x)",
                   correct_checksum);

	    if(statistic)
		    stats.cr.icmpv4++;
    }
  }
  
  apsr_output("\n");
 
  switch(icmpv4_header->type) {
	  case APSR_ICMP_ECHO_REPLY:
          case APSR_ICMP_ECHO_REQUE:
          case APSR_ICMP_INFO_REPLY:
          case APSR_ICMP_INFO_REQUE:
          case APSR_ICMP_MASK_REQUE:
          case APSR_ICMP_MASK_REPLY:
   	    apsr_output("  Identifier: %d, Seq.: %d\n", 
			    apsr_ntohs(icmpv4_header->ext.echo.identifier), 
			    apsr_ntohs(icmpv4_header->ext.echo.seq_num));
	    break;
	  case APSR_ICMP_SECU_FAILU:
	    apsr_output("  Pointer: %d\n", apsr_ntohs(icmpv4_header->ext.secu.pointer));
	    break;
	  case APSR_ICMP_TIME_REQUE:
	  case APSR_ICMP_TIME_REPLY:
            apsr_output("  Identifier: %d, Seq.: %d\n", 
			    apsr_ntohs(icmpv4_header->ext.timestamp.identifier), 
			    apsr_ntohs(icmpv4_header->ext.timestamp.seq_num));
	    apsr_output("  Originate Timestamp: %d, Receive Timestamp: %d\n",
			    apsr_ntohl(icmpv4_header->ext.timestamp.originate),
			    apsr_ntohl(icmpv4_header->ext.timestamp.receive));
	    apsr_output("  Transmit Timestamp: %d\n",
			    apsr_ntohl(icmpv4_header->ext.timestamp.transmit));
	    break;
	  case APSR_ICMP_PARA_PROBL:
	    apsr_output("  Pointer: %d\n",(int) icmpv4_header->ext.para.pointer);
	  case APSR_ICMP_REDIRECT:
            gateway_ip = strdup(inet_ntoa(icmpv4_header->ext.gateway_ip));
	    apsr_output("  Gateway IP: %s\n", gateway_ip);
	    if(verbose) {
/*            resolve_ip(gateway_ip, AF_INET); */
	    }
	    free(gateway_ip);
	    break;
	  case APSR_ICMP_DOMAIN_REQ:
            apsr_output("  Identifier: %d, Seq.: %d\n",
                       apsr_ntohs(icmpv4_header->ext.dns.identifier),
                       apsr_ntohs(icmpv4_header->ext.dns.seq_num));
            break;
          case APSR_ICMP_DOMAIN_REP:
            apsr_output("  Identifier: %d, Seq.: %d\n",
                       apsr_ntohs(icmpv4_header->ext.dns.identifier),
                       apsr_ntohs(icmpv4_header->ext.dns.seq_num));
	    apsr_output("  Time-To-Live: %d\n", 
		       apsr_ntohl(icmpv4_header->ext.dns.ttl));
            break;
	  default:
	    break;
  }

  if(statistic) 
          stats.c.icmpv4++;
  
  return(RET_SUCCESS);
}
