#include "../include/aprecv-general.h"

static const char *vrrpauth[] = {
	" (No authentication)",
	" (Simple text password)",
	" (IP Authentication Header)",
};

int print_vrrp_packet (unsigned char *packet, int opt_size) {
  static struct vrrp_pkt_header *vrrp_header;
  static unsigned short version, type, i;
  static unsigned char *buf;

  vrrp_header = (struct vrrp_pkt_header *)(packet+data_link_size+opt_size);
  
  version = APSR_VRRP_VERSION(vrrp_header);
  type    = APSR_VRRP_TYPE(vrrp_header);

  apsr_output("- VRRP Header -\n  Version: %d, Type: %d",
		  (int) version,(int) type);
  if(verbose) {
    switch(type) {
	case APSR_VRRP_TYPE_ADVERTISEMENT:
          apsr_output(" (Advertisement)");
	  break;
	default:
	  apsr_output(" (unknown)");
	  break;
    }
  }

  apsr_output("\n  Virtual Router ID: %d, Priority: %d",
		  (int) vrrp_header->vrid, vrrp_header->priority);
  
  apsr_output("\n  Count IP Addrs: %d, Auth. type: %d",
		  (int) vrrp_header->count_ip, (int) vrrp_header->auth_type);
  if(verbose) 
	  apsr_output("%s", vrrpauth[(int) vrrp_header->auth_type]);

  apsr_output("\n  Checksum: 0x%04x\n", apsr_ntohs(vrrp_header->sum));

  buf = packet+data_link_size+opt_size+8;  
  if(vrrp_header->count_ip > 0) {
	  apsr_output("  IP Addrs: ");
	  for(i = 0; i < vrrp_header->count_ip; i++) {
		  apsr_output("%u.%u.%u.%u ", buf[0], buf[1], buf[2], buf[3]);
		  buf += 4;
	  } 
	  apsr_output("\n");
  }  
  if(vrrp_header->auth_type == APSR_VRRP_AUTHTYPE_SIMPLE) {
	  apsr_output("  Password: \"");
	  for(i = 0; i <= 7; i++) {
		  if(isprint(buf[i]))
			apsr_output("%c");
		  else
		  	apsr_output(".");
          }			
	  apsr_output("\"\n");
  }		  	
  
  if(statistic) 
          stats.c.vrrp++;
  
  return(RET_SUCCESS);
}
