#include "../include/aprecv-general.h"
#include "aprecv-iptrace.h"

void read_iptrace(int max_packets) {
	size_t read_bytes;
	unsigned char name[11], iptrace_header[40];
	unsigned char *packet;
	struct iptrace_v2_pkt_hdr *iptrace_v2_header;
	struct aprecv_frame_header aprecv_header;
	unsigned int file_size, sec, pkt_len;
	int count = 0;

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        int *gzerror_num=NULL;
        #endif
		
	#ifdef APRECV_DEBUG
	unsigned int i;
	#endif

	apsr_output("Read packet(s) from file: %s.\n", iptrace_trace_file);

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        if((file_handle = gzopen(iptrace_trace_file, "r")) == NULL) {
	#else
	if((file_handle = fopen(iptrace_trace_file, "r")) == NULL) {
	#endif
		apsr_stream_output((FILE *) stderr, "Error: Can't open iptrace file %s.\n", iptrace_trace_file);
		#ifdef APRECV_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n", 
				strerror(errno));
		#endif
                exit(EXIT_FAILURE);
        }
	
        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        gzseek(file_handle, 0, SEEK_END);
        file_size = gztell(file_handle);

        /* Replacement for rewind */
        /* !!!FIXME!!! - gzseek seems not to set SEEK_SET */
        (void) gzseek(file_handle, 0L, SEEK_SET);
        #else
        fseek(file_handle, 0, SEEK_END);
        file_size = ftell(file_handle);

        /* Replacement for rewind */
        (void) fseek(file_handle, 0L, SEEK_SET);
        #endif

	#ifdef APRECV_DEBUG
  	apsr_output("Iptrace file size is %d bytes.\n", file_size);
	#endif

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        read_bytes = gzread(file_handle, name, 11);
        #else
        read_bytes = fread(name, 1, 11, file_handle);
        #endif

	if(read_bytes != 11)
	        ERROR_EXIT("Error occured while Iptrace version reading.\n");
	
        name[11] = '\0';

	#ifdef APRECV_DEBUG
	apsr_output("Check for iptrace version.\n");
	#endif
        if (strncmp(name, "iptrace 1.0", 10) == 0) {
          apsr_output("File type is iptrace version 1.0. Not supported @ the moment!\n");
        }
        else if (strncmp(name, "iptrace 2.0", 10) == 0) {
          apsr_output("File type is iptrace version 2.0.\n");
          while(1) {
            #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                  if((gztell(file_handle) == file_size)) {
	            #else
	            if((ftell(file_handle) == file_size)) {
	              #endif
	              apsr_output("\nRead iptrace dump file completed.\n");
                goto close_it;
            }
    
            count++;

            #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
            read_bytes = gzread(file_handle, iptrace_header, 40);
            #else
            read_bytes = fread(iptrace_header, 1, 40, file_handle);
            #endif
            
	    if(read_bytes <= 0) {
              #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
              if(gzeof(file_handle))
              #else
              if(feof(file_handle)) 
              #endif
		      ERROR_EXIT("End of file reached, but not at the right position, maybe truncated.\n");

              #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
              if(gzerror(file_handle, gzerror_num))
		      apsr_output("Error: Read packet failed(%d).\n",
				      gzerror_num);
	      #else
	      if(ferror(file_handle))
		      apsr_output("Error: Read packet failed.\n");
	      #endif
              goto close_it;
            }
            else {
	      iptrace_header[40] = '\0';
	      iptrace_v2_header = (struct iptrace_v2_pkt_hdr *)(iptrace_header);
	      #ifdef APRECV_DEBUG
	      apsr_output("Interface name: %s\n", iptrace_v2_header->if_name);
	      apsr_output("Interface description: \"");
	      for(i = 0; i <= 11; i++) 
		      apsr_output("%c", iptrace_v2_header->if_desc[i]);
	      apsr_output("\"\n");
	      apsr_output("Interface type: 0x%x\n", iptrace_v2_header->if_type);
	      apsr_output("Tx flag: 0x%x\n", iptrace_v2_header->tx_flag);
	      #endif
	      pkt_len = apsr_ntohl(iptrace_v2_header->pkt_length)-32;
	      if(pkt_len <= 0) {
		#ifdef APRECV_DEBUG
		apsr_output("Error: pkt_len of iptrace header is too small, skipp packet.\n");
		#endif
      	      }
	      else {
                #ifdef APRECV_DEBUG
	        apsr_output("Packet length: %u\n", pkt_len);
  	        #endif
		packet = (unsigned char *) xmalloc(pkt_len+1);

                #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                read_bytes = gzread(file_handle, packet, (size_t) pkt_len);
                #else
                read_bytes = fread(packet, 1,(size_t) pkt_len, file_handle);
                #endif
		
                if(read_bytes <= 0) {
                  #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                  if(gzeof(file_handle))
                  #else
                  if(feof(file_handle))
                  #endif
			  ERROR_EXIT("End of file reached, but not at the right position, maybe truncated.\n");

                  #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                  if(gzerror(file_handle, gzerror_num))
	          #else
	          if(ferror(file_handle))
		  #endif
			  apsr_output("Error: Read packet failed.\n");
                  goto close_it;
                }
                else {
		  stats.c.all++;
			
 		  sec = iptrace_v2_header->tv_sec % 86400;
		  apsr_output("\n%02d:%02d:%02d - Read frame %lu(%u bytes length)...\n", sec / 3600, (sec % 3600) / 60, sec % 60, stats.c.all, pkt_len);
		  	 
	          aprecv_header.caplen = pkt_len; 
                  aprecv_header.len    = pkt_len;
		  switch(iptrace_v2_header->if_type) {
		    case IPTRACE_IF_TYPE_TOKENRING:
		      data_link_size = 14;
		      print_token_packet(&aprecv_header, packet, 0);
	              break;
		    case IPTRACE_IF_TYPE_ETHERNET:
		    case IPTRACE_IF_TYPE_8023:
		      data_link_size = 14;
		      print_ethernet_packet(&aprecv_header, packet, 0);
		      break;
		    case IPTRACE_IF_TYPE_LOOP:
		      data_link_size = 4;
		      print_null_packet(&aprecv_header, packet, 0);
		      break;
		    default:
		      apsr_output("\n- Not support packet type -\n");
		      break;
	          }
                }
	      } 
              if(count == max_packets) {
		apsr_output("\nRead iptrace dump file completed.\n");
		goto close_it;
              }
            }
          }
	}
        else {
		apsr_output("Unknown iptrace file format.\n");
        }

close_it:

	/* Ugly stuff from src/error.c. But i don't know how to work with
	 * src/error.c - this gives a SIGSEGV ! */
	
        if(statistic)
		print_statistic();

	if(iptrace_trace_file) {
                #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                if(gzclose(file_handle) < 0) {
	        #else
	        if(fclose(file_handle) < 0) {
		#endif
			apsr_stream_output((FILE *) stderr, "Error: Can't close tracefile %s.\n", iptrace_trace_file);
                        #ifdef APRECV_DEBUG
                        apsr_stream_output((FILE *) stderr, "Error code: %s.\n", strerror(errno));
                        #endif
		}
        }

	#ifdef HAVE_APSRLIB_MODULE_SUPPORT
	close_modules();
	#endif

	if(server_addr)
		close_server_connection();

	exit(EXIT_SUCCESS);
}
