/* ARP internals */
#define APSR_MAX_ADDR_LEN	7	/* Maximal address length  */


/* ARP protocol opcodes */
#define APSR_ARPOP_REQUEST	1	/* ARP request.  */
#define APSR_ARPOP_REPLY	2	/* ARP reply.  */
#define APSR_ARPOP_RREQUEST	3	/* RARP request.  */
#define APSR_ARPOP_RREPLY	4	/* RARP reply.  */
#define APSR_ARPOP_InREQUEST	8	/* InARP request.  */
#define APSR_ARPOP_InREPLY	9	/* InARP reply.  */
#define APSR_ARPOP_NAK		10	/* (ATM)ARP NAK.  */

#define APSR_ARPOP_INREQUEST	APSR_ARPOP_InREQUEST	/* See above. */
#define APSR_ARPOP_INREPLY	APSR__InREPLY		/* See above. */


/* ARP protocol HARDWARE identifiers. */
#define APSR_ARPHRD_NETROM	0	/* From KA9Q: NET/ROM pseudo. */
#define APSR_ARPHRD_ETHER	1	/* Ethernet 10/100Mbps.  */
#define APSR_ARPHRD_EETHER	2	/* Experimental Ethernet.  */
#define APSR_ARPHRD_AX25     	3	/* AX.25 Level 2.  */
#define APSR_ARPHRD_PRONET   	4	/* PROnet token ring.  */
#define APSR_ARPHRD_CHAOS    	5	/* Chaosnet.  */
#define APSR_ARPHRD_IEEE802  	6	/* IEEE 802.2 Ethernet/TR/TB.  */
#define APSR_ARPHRD_ARCNET   	7	/* ARCnet.  */
#define APSR_ARPHRD_APPLETLK 	8	/* APPLEtalk.  */
#define APSR_ARPHRD_DLCI     	15	/* Frame Relay DLCI.  */
#define APSR_ARPHRD_ATM      	19	/* ATM.  */
#define APSR_ARPHRD_METRICOM 	23	/* Metricom STRIP (new IANA id).  */


/* Dummy types for non ARP hardware */
#define APSR_ARPHRD_SLIP	256	/* Serial over IP (SLIP) */
#define APSR_ARPHRD_CSLIP	257	/* Compressed (?) SLIP */
#define APSR_ARPHRD_SLIP6	258	/* Serial over IPv6 (SLIP6) */
#define APSR_ARPHRD_CSLIP6	259	/* Compressed (?) SLIP6  */
#define APSR_ARPHRD_RSRVD	260	/* Notional KISS type.  */
#define APSR_ARPHRD_ADAPT	264	/* ??? */
#define APSR_ARPHRD_ROSE	270	/* ??? */
#define APSR_ARPHRD_X25		271	/* CCITT X.25.  */
#define APSR_ARPHDR_HWX25	272	/* Boards with X.25 in firmware.  */
#define APSR_ARPHRD_PPP		512	/* Point-to-point protocol (PPP) */
#define APSR_ARPHRD_HDLC	513	/* (Cisco) HDLC.  */
#define APSR_ARPHRD_LAPB 	516	/* LAPB.  */
#define APSR_ARPHRD_DDCMP	517	/* Digital's DDCMP.  */
#define APSR_ARPHRD_TUNNEL	768	/* IPIP tunnel.  */
#define APSR_ARPHRD_TUNNEL6	769	/* IPIP6 tunnel.  */
#define APSR_ARPHRD_FRAD	770	/* Frame Relay Access Device.  */
#define APSR_ARPHRD_SKIP	771	/* SKIP vif.  */
#define APSR_ARPHRD_LOOPBACK	772	/* Loopback device.  */
#define APSR_ARPHRD_LOCALTLK	773	/* Localtalk device.  */
#define APSR_ARPHRD_FDDI	774	/* Fiber Distributed Data Interface. */
#define APSR_ARPHRD_BIF		775	/* AP1000 BIF.  */
#define APSR_ARPHRD_SIT		776	/* sit0 device - IPv6-in-IPv4.  */
#define APSR_ARPHRD_IPDDP 	777	/* IP-in-DDP tunnel.  */
#define APSR_ARPHRD_IPGRE	778	/* GRE over IP.  */
#define APSR_ARPHRD_PIMREG	779	/* PIMSM register interface.  */
#define APSR_ARPHRD_HIPPI	780	/* High Performance Parallel I'face. */
#define APSR_ARPHRD_ASH   	781	/* (Nexus Electronics) Ash.  */
#define APSR_ARPHRD_ECONET	782	/* Acorn Econet.  */
#define APSR_ARPHRD_IRDA	783	/* Linux-IrDA.  */
#define APSR_ARPHRD_FCPP	784	/* Point to point fibrechanel.  */
#define APSR_ARPHRD_FCAL	785	/* Fibrechanel arbitrated loop.  */
#define APSR_ARPHRD_FCPL	786	/* Fibrechanel public loop.  */
#define APSR_ARPHRD_FCPFABRIC	787	/* Fibrechanel fabric.  */
#define APSR_ARPHRD_IEEE802_TR	800	/* Magic type ident for TR.  */

/* ARP Flag values.  */
#define APSR_ATF_COM		0x02	/* Completed entry (ha valid).  */
#define APSR_ATF_PERM		0x04	/* Permanent entry.  */
#define APSR_ATF_PUBL		0x08	/* Publish entry.  */
#define APSR_ATF_USETRAILERS	0x10	/* Has requested trailers.  */
#define APSR_ATF_NETMASK	0x20	/* Want to use a netmask (only
                                           for proxy entries). */
#define APSR_ATF_DONTPUB	0x40	/* Don't answer this addresses.  */
#define APSR_ATF_MAGIC		0x80	/* Automatically added entry.  */

#define APSR_ARP_HDR_LEN	8	/* ARP header length */

#define APSR_ARP_REQUEST   	1               /* ARP request	 	*/
#define APSR_ARP_REPLY     	2               /* ARP reply		*/
#define APSR_ARP_RREQUEST  	3               /* Reverse ARP request	*/
#define APSR_ARP_RREPLY    	4               /* Reverse ARP reply	*/
#define APSR_ARP_DRREQUEST	5		/* Dynamic Reverse ARP request	*/
#define APSR_ARP_DRREPLY	6               /* Dynamic Reverse ARP reply	*/
#define APSR_ARP_DRERROR	7               /* Dynamic Reverse ARP error	*/
#define APSR_ARP_InREQUEST 	8               /* Inverse ARP request	*/
#define APSR_ARP_InREPLY   	9               /* Inverse ARP reply	*/
#define APSR_ARP_NAK       	10              /* (ATM) ARP NAK	*/
#define APSR_ARP_MARSREQUEST	11		/* MARS-Request		*/
#define APSR_ARP_MARSMULTI	12              /* MARS-Multi		*/
#define APSR_ARP_MARSMSERV	13              /* MARS-MServ		*/
#define APSR_ARP_MARSJOIN	14              /* MARS-Join		*/
#define APSR_ARP_MARSLEAVE	15              /* MARS-Leave		*/
#define APSR_ARP_MARSNAK	16              /* MARS-Nak		*/
#define APSR_ARP_MARSUNSERV	17              /* MARS-UnServ		*/
#define APSR_ARP_MARSSJOIN	18              /* MARS-SJoin		*/
#define APSR_ARP_MARSSLEAVE	19              /* MARS-SLeave		*/
#define APSR_ARP_MARSGRREQUE	20              /* MARS-Grouplist-Request	*/
#define APSR_ARP_MARSGRREPLY	21              /* MARS-Grouplist-Reply	*/
#define APSR_ARP_MARSREDMAP	22              /* MARS-Redirect-Map	*/
#define APSR_ARP_MAPOSUNARP	23              /* MAPOS-UNARP		*/

#define APSR_ARP_HWTYPE_E10	1		/* Ethernet 10 MBit/FDDI	*/
#define APSR_ARP_HWTYPE_EE	2		/* Experimental Ethernet 3 Mbit	*/
#define	APSR_ARP_HWTYPE_AX25	3		/* Amateur Radio AX.25	*/
#define APSR_ARP_HWTYPE_PTR	4		/* Proteon ProNet Token Ring	*/
#define APSR_ARP_HWTYPE_CHS	5		/* Chaos		*/
#define	APSR_ARP_HWTYPE_IEEE	6		/* IEEE 802.x networks	*/
#define	APSR_ARP_HWTYPE_ARC	7		/* ARCNet		*/
#define APSR_ARP_HWTYPE_HCH	8		/* HyperChannel		*/
#define APSR_ARP_HWTYPE_LANS	9		/* LanStar		*/
#define APSR_ARP_HWTYPE_ASA	10		/* Autonet Short Address*/
#define APSR_ARP_HWTYPE_LTL	11		/* LocalTalk		*/
#define APSR_ARP_HWTYPE_LN	12		/* LocalNet (IBM PCNET/SYSTEK)	*/
#define APSR_ARP_HWTYPE_UL	13		/* Ultra Link		*/
#define	APSR_ARP_HWTYPE_SMDS	14		/* SMDS			*/
#define	APSR_ARP_HWTYPE_FR	15		/* FrameRelay		*/
#define	APSR_ARP_HWTYPE_ATM1	16		/* ATM			*/
#define APSR_ARP_HWTYPE_HDLC	17		/* HDLC			*/
#define APSR_ARP_HWTYPE_FC	18              /* FiberChannel		*/
#define APSR_ARP_HWTYPE_ATM2	19              /* ATM			*/
#define APSR_ARP_HWTYPE_SLIP	20              /* Serial Line		*/
#define APSR_ARP_HWTYPE_ATM3	21              /* ATM			*/

