#include "../include/apsrlib-config.h"

/* Create a CIP packet. */
int apsr_cip (unsigned short int cpf, unsigned short int type,
	      unsigned short int len_address_data, unsigned int identifier,
	      unsigned int sequence, unsigned short int data_type,
	      unsigned short int len, unsigned char *data,
	      const unsigned char *payload, int payload_size,
	      unsigned char *buf)
{
	static struct cip_pkt_header cip_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
          ERROR("apsr_cip: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	cip_hdr.cpf			= htons(cpf);
	cip_hdr.type			= htons(type);
	cip_hdr.len_address_data	= htons(len_address_data);
	cip_hdr.identifier		= identifier;
	cip_hdr.sequence		= sequence;
	cip_hdr.data_type		= htons(data_type);
	cip_hdr.len			= htons(len);

	if(data != NULL) {
		cip_hdr.data = xmalloc(sizeof(data));
		memcpy(cip_hdr.data, data, sizeof(data));
	}
	
	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_CIP + sizeof(data), payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&cip_hdr,
	       APSR_CIP);

	return(RET_SUCCESS);
}

