#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"atmp"
#define MODULE_DESCRIPTION	"Example ATMP (Ascend Tunnel Management Protocol) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	5150	
#define MODULE_IDENT_ID2	5150

#define MODULE_FILTER		"udp and port 5150"

static const char *mod_atmp_type[]= {
	" (Unknown)",
	" (Register request)",
	" (Challenge request)",
	" (Challenge reply)",
	" (Register reply)",
	" (Deregister request)",
	" (Deregister reply)",
	" (Error notification)",
};

struct mod_atmp_header {
	unsigned char version;
	unsigned char type;
	unsigned short identifier;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
	module_info.proto1      = MODULE_IDENT_PN1;
	module_info.proto2      = 0;
	module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
					
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_atmp_header *atmp_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    atmp_header = (struct mod_atmp_header *)buf;
	    apsr_output("- ATMP Header -\n");
	    apsr_output("  Version: %u\n", atmp_header->version);
	    apsr_output("  Type: %u", atmp_header->type);
	    if(verbose) 
		    apsr_output("%s", mod_atmp_type[atmp_header->type]);
            apsr_output("\n  Identifier: %u\n", atmp_header->identifier);
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
