#include <apsrlib/apsrlib.h>

/* TODO: Print CRC */

/* The functions map_to_long(msrand), mrand and getkey is copied 
 * from bo121unix and renamed. */

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"backorifice"
#define MODULE_DESCRIPTION	"Example Back Orifice protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP

/* Note: The default Back Orifice Port ist 31337, but the server can change it.
 */

#define MODULE_IDENT_ID1	31337	
#define MODULE_IDENT_ID2	31337

#define MODULE_FILTER		"udp and port 31337"

/* #define APSR_BACKORIFICE_DEFAULT_MAGIC 	0x2A212A515754593F */

/* Global */
long holdrand = 1L;
char password[256]; 	/* Can be changed */

struct mod_backorifice_header {
	unsigned char magic[8];
	int length;
	int id;
	char type;
	/* unsigned char *data;
	 * unsigned char crc; */
};

static const char *mod_backorifice_type[]= {
	" (Error)",
	" (Ping)",
	" (System reboot)",
	" (System lock up)",
	" (List system passwords)",
	" (View console)",
	" (Get system information)",
	" (Log pressed keys)",
	" (Send keypress log)",
	" (Show a dialog box)",
	" (Delete an value from the registry)",
	" (Create TCP redirection)",
	" (Delete TCP redirection)",
	" (List TCP redirections)",
	" (Start application)",
	" (End application)",
	" (Export a share resource)",
	" (Cancel share export)",
	" (Show export list)",
	" (Resend packet)",
	" (Enable HTTP server)",
	" (Disable HTTP server)",
	" (Resolve host name)",
	" (Compress a file)",
	" (Uncompress a file)",
	" (Plug-in execute)",
	" (Show active processes)",
	" (Kill a process)",
	" (Start a process)",
	" (Create a key in the registry)",
	" (Set the value of a key in the registry)",
	" (Delete a key in the registry)",
	" (Enumerate registry keys)",
	" (Enumerate registry value)",
	" (Capture a static image)",
	" (Capture a video stream)",
	" (Play a sound file)",
	" (Show available image/video capture devices)",
	" (Capture the screen to a file)",
	" (Start sending a file using TCP)",
	" (Start receiving a file using TCP)",
	" (List (running) plug-ins)",
	" (Kill plug-in)",
	" (List directory)",
	" (Find a file)",
	" (Delete a file)",
	" (View file contents)",
	" (Rename a file)",
	" (Copy a file)",
	" (List all network devices, domain names and shares)",
	" (Connect a network resource)",
	" (End connection of a network resource)",
	" (Show network connections)",
	" (Create directory)",
	" (Remove directory)",
	" (Show running applications)",
};		

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

void map_to_long (unsigned int seed) {
	holdrand = (long)seed;
}

int mrand (void) {
	return(((holdrand = holdrand * 214013L + 2531011L) >> 16) & 0x7fff);
}

unsigned int getkey() {
	int x, y;
	unsigned int z;

	y = strlen(password);
	if (!y)
		return 31337;
	else {
		z = 0;
        for (x = 0; x < y; x++)
		z+= password[x];
                for (x = 0; x < y; x++) {
			if (x%2)
				z-= password[x] * (y-x+1);
			else
			 	z+= password[x] * (y-x+1);
			z = z%RAND_MAX;
	        }
		z = (z * y)%RAND_MAX;
		
		return z;
        }
}		  

extern char verbose;
extern unsigned short module_length;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_backorifice_header *bo_header;
  int y = 0;

  /* Default password is == nothing */
  password[0] = 0;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {

  	    map_to_long(getkey());
	    for (y = 0; y < module_length; y++)
		    buf[y] = buf[y] ^ (mrand()%256);
  
	    bo_header = (struct mod_backorifice_header *)buf;
	    apsr_output("- Back Orifice Header -\n");

	    apsr_output("  Magic: \"");
	    for(y = 0; y <= 7; y++) {
		    if(isprint(bo_header->magic[y]))
			    apsr_output("%c", bo_header->magic[y]);
		    else
			    apsr_output(".");
	    }
	    apsr_output("\"\n  Length: %u, ID: 0x%08x\n",
	    	    bo_header->length, bo_header->id);
	    apsr_output("  Type: %u", bo_header->type);
	    if(verbose)
		    apsr_output("%s", mod_backorifice_type[(int)bo_header->type]);
	    apsr_output("\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
