#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"masc"
#define MODULE_DESCRIPTION	"Example Multicast Address-Set Claim sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	2587
#define MODULE_IDENT_ID2	2587

#define MODULE_FILTER		"tcp and port 2587"

static const char *masc_types[]= {
	" (Unknown)",
	" (Open)",
	" (Update)",
	" (Notification)",
	" (Keepalive)",
};

struct mod_masc_header {
	unsigned short length;
	unsigned char type;
	unsigned char reserved;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_masc_header *masc_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    masc_header = (struct mod_masc_header *)buf;
	    apsr_output("- MASC Header -\n");
	    apsr_output("  Length: %u\n", apsr_ntohs(masc_header->length));
	    apsr_output("  Type: %u", masc_header->type);
	    if(verbose)
		    apsr_output("%s", masc_types[masc_header->type]);
	    apsr_output("\n  Reserved: %u\n", masc_header->reserved);
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
