#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"Mobile IP"
#define MODULE_DESCRIPTION	"Example Mobile IP sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	434
#define MODULE_IDENT_ID2	434

#define MODULE_FILTER		"udp and port 434"

#define APSR_MOBILEIP_REQREQ	1
#define APSR_MOBILEIP_REGREP	3
#define APSR_MOBILEIP_REGUPDATE	20
#define APSR_MOBILEIP_REGACK	21

struct mod_mobileip_header {
	unsigned char type;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_mobileip_header *mobileip_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    mobileip_header = (struct mod_mobileip_header *)buf;
	    apsr_output("- Mobile IP Header -\n");
	    apsr_output("  Type: %u", mobileip_header->type);
	    if(verbose) {
		    switch(mobileip_header->type) {
			    case APSR_MOBILEIP_REQREQ:
		              apsr_output(" (Registration request)");
			      break;
			    case APSR_MOBILEIP_REGREP:
			      apsr_output(" (Registration reply)");
			      break;
			    case APSR_MOBILEIP_REGUPDATE:
			      apsr_output(" (Registration update)");
			      break;
			    case APSR_MOBILEIP_REGACK:
			      apsr_output(" (Registration acknowledgement)");
			      break;
			    default:
			      apsr_output(" (Unknown)");
			      break;
		    }
	    }  
	    apsr_output("\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
