#include <apsrlib/apsrlib.h>

#define MODULE_VERSION          "0.01"
#define MODULE_NAME             "ntp"
#define MODULE_DESCRIPTION      "Network time protocol (NTP) sniffer"
#define MODULE_IDENT_FAMILY     AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	123
#define MODULE_IDENT_ID2        123

#define MODULE_FILTER		"(tcp or udp) and port 123"

/* TODO: Sync distance						*/
/* TODO: Sync dispersion					*/
/* TODO: Reference clock identifier (RCI)			*/
/* TODO: Reference timestamp (64 bits)				*/
/* TODO: Originate timestamp (64 bits)				*/
/* TODO: Receive timestamp (64 bits)				*/
/* TODO: Transmit timestamp (64 bits)				*/
/* TODO: Time output functions					*/
/* TODO: Time delta functions					*/
/* TODO: Floating point calculations. Not very portable.	*/


/* Macros for the NTP status */
#define APSR_NTP_LI(mod_ntp_header)	((mod_ntp_header)->li_vn_mode & 0xc0)
#define APSR_NTP_VN(mod_ntp_header)	(((mod_ntp_header)->li_vn_mode & 0x37) >> 3)
#define APSR_NTP_MODE(mod_ntp_header)((mod_ntp_header)->li_vn_mode & 0x07)


/* NTP leap codes */
#define APSR_NTP_NO_WARNING		0x00
#define APSR_NTP_PLUS_SECOND		0x40
#define APSR_NTP_MINUS_SECOND	0x80
#define APSR_NTP_ALARM		0xc0

/* NTP codes */
#define APSR_NTP_MODE_UNSPEC		0
#define APSR_NTP_MODE_S_ACT		1
#define APSR_NTP_MODE_S_PAS		2
#define APSR_NTP_MODE_CLIENT		3
#define APSR_NTP_MODE_SERVER		4
#define APSR_NTP_MODE_BROADCAST	5
#define APSR_NTP_MODE_RES1		6
#define APSR_NTP_MODE_RES2		7


struct mod_ntp_header {
	unsigned char	li_vn_mode;
	unsigned char	stratum;
	unsigned char	ppoll;
	unsigned int	precision:8;
	
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version     = MODULE_VERSION;
	module_info.name        = MODULE_NAME;
	module_info.desc        = MODULE_DESCRIPTION;
	module_info.bpf_filter  = MODULE_FILTER;
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,                 unsigned short id1, unsigned short id2)
{
  struct mod_ntp_header *ntp_hdr;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	ntp_hdr = (struct mod_ntp_header *) buf;

	apsr_output("- NTP header -\n");
	apsr_output("  Version: %u\n", APSR_NTP_VN(ntp_hdr));

	switch (APSR_NTP_LI(ntp_hdr)) {
		case APSR_NTP_NO_WARNING:
			break;
		case APSR_NTP_PLUS_SECOND:
			apsr_output("  LI: +1 second");
			if(verbose)
			  apsr_output("(following minute has 61 seconds)");
			apsr_output("\n");
			break;
		case APSR_NTP_MINUS_SECOND:
			apsr_output("  LI: -1 second");
			if(verbose)
			  apsr_output("(following minute has 59 seconds)");
			apsr_output("\n");
			break;
	};

	apsr_output("  Mode: ");

	switch (APSR_NTP_MODE(ntp_hdr)) {
		case APSR_NTP_MODE_UNSPEC:
			apsr_output("Unspecified\n");
			break;
		case APSR_NTP_MODE_S_ACT:
			apsr_output("Symmetric active\n");
			break;
		case APSR_NTP_MODE_S_PAS:
			apsr_output("Symmetric passive\n");
			break;
		case APSR_NTP_MODE_CLIENT:
			apsr_output("Client\n");
			break;
		case APSR_NTP_MODE_SERVER:
			apsr_output("Server\n");
			break;
		case APSR_NTP_MODE_BROADCAST:
			apsr_output("Broadcast\n");
			break;
		case APSR_NTP_MODE_RES1:
			apsr_output("Reserved 1\n");
			break;
		case APSR_NTP_MODE_RES2:
			apsr_output("Reserved 2\n");
			break;
	}

	apsr_output("  Stratum: %d\n", ntp_hdr->stratum);
	apsr_output("  Poll: %d\n", ntp_hdr->ppoll);
	apsr_output("  Precision: %d\n\n", ntp_hdr->precision);
  } 

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		     family, protocol_number, id1, id2);
  #endif

  return(RET_SUCCESS);
}

